@@ -12,7 +12,7 @@ class Component(object):
     generic_card_type_widget, ui_component, controller, main_widget,
     review_controller, review_widget, file format, plugin, hook,
     criterion, criterion_applier, statistics_page, sync_server,
-    all the abstract dialogs, ...
+    study_mode, all the abstract dialogs, ...
 
     'used_for' can store certain relationships between components, e.g.
     a card type widget is used for a certain card type.
@@ -22,10 +22,16 @@ class Component(object):
     time on a mobile device, and should be done lazily.) Only the main
     widget is stored as an instance here.
 
-    When 'instantiate == LATER', the component is lazily created when needed.
+    To achieve this lazy instantiation for widget, set 'instantiate == LATER'.
+    Other components can then instantiate the widget when they see fit.
     The instance is not cached for subsequent reuse, as these widgets
     typically can become obsolete/overwritten by plugins.
 
+    It can be that when instantiating a component, not all the other components
+    on which it relies have been instantiated yet. E.g., the log and the
+    database depend on each other before doing actual work. Therefore, some of
+    the  actual initialisation work can be postponed to the 'activate' function.
+
     Each component has access to all of the context of the other components
     because it hold a reference to the user's component manager.
 
@@ -34,6 +40,9 @@ class Component(object):
     This means that derived components should always call the
     Component.__init__ if they provide their own constructor.
 
+    In case the GUI needs to add functionality to a certain component, that
+    can be done through component_manager.add_gui_to_component().
+
     """
 
     component_type = ""
@@ -44,8 +53,11 @@ class Component(object):
 
     instantiate = IMMEDIATELY
 
-    def __init__(self, component_manager):
+    def __init__(self, component_manager, **kwds):
+        super().__init__(**kwds)  # For parent classes other than 'Object'.
         self.component_manager = component_manager
+        self.gui_components = []
+        self.instantiated_gui_components = []
 
     def activate(self):
 
@@ -53,12 +65,22 @@ class Component(object):
         work, and which can't happen in the constructor, e.g. because
         components on which it relies have not yet been registered.
 
+        GUI classes are only instantiated when activated, since that can take
+        a lot of time on mobile clients.
+
         """
 
-        pass
+        for component in self.gui_components:
+            component = component(component_manager=self.component_manager)
+            self.component_manager.register(component)
+            component.activate()
+            self.instantiated_gui_components.append(component)
 
     def deactivate(self):
-        pass
+        for component in self.instantiated_gui_components:
+            component.deactivate()
+            self.component_manager.unregister(component)
+        self.instantiated_gui_components = []
 
     # Convenience functions, for easier access to all of the context of
     # libmnemosyne from within a component.
@@ -88,15 +110,7 @@ class Component(object):
         return self.component_manager.current("main_widget")
 
     def review_widget(self):
-
-        """Apart from the main widget, we create all other widgets lazily for
-        efficiency reasons. The review widget instance is therefore not stored
-        in the component manager, but is under the control of the review
-        controller.
-
-        """
-
-        return self.review_controller().widget
+        return self.component_manager.current("review_widget")
 
     def controller(self):
         return self.component_manager.current("controller")
@@ -111,14 +125,14 @@ class Component(object):
         return self.component_manager.card_type_with_id[id]
 
     def render_chain(self, id="default"):
-        return self.component_manager.render_chain_by_id[id]
+        return self.component_manager.render_chain_with_id[id]
+
+    def study_mode_with_id(self, id):
+        return self.component_manager.study_mode_with_id[id]
 
     def plugins(self):
         return self.component_manager.all("plugin")
 
-    def start_review(self):
-        self.review_controller().reset()
-
     def flush_sync_server(self):
 
         """If there are still dangling sessions (i.e. those waiting in vain
@@ -127,7 +141,7 @@ class Component(object):
         change the database (e.g. adding a card). Otherwise, if these
         sessions close later during program shutdown, their backup
         restoration will override the changes.
-        
+
         Also stop any running media.
 
         """
