@@ -6,7 +6,6 @@ import os
 import sys
 import copy
 import time
-import shutil
 
 from mnemosyne.libmnemosyne.fact import Fact
 from mnemosyne.libmnemosyne.translator import _
@@ -34,10 +33,11 @@ class DefaultController(Controller):
     """
 
     def activate(self):
+        self.study_mode = None
         Controller.activate(self)
         self.next_rollover = self.database().start_of_day_n_days_ago(n=-1)
 
-    def heartbeat(self):
+    def heartbeat(self, db_maintenance=True):
 
         """Making sure, even if the user leaves the program open indefinitely,
         that backups get taken, that the cards scheduled for the day get dumped
@@ -47,24 +47,39 @@ class DefaultController(Controller):
         """
 
         if time.time() > self.next_rollover:
-            self.flush_sync_server()
-            if not self.database().is_loaded() or \
-                not self.database().is_accessible():
-                # Make sure we don't continue if e.g. the GUI or another thread
-                # holds the database
-                return
-            self.database().backup()
-            self.log().saved_database()
-            self.log().loaded_database()
-            self.log().future_schedule()
-            self.log().dump_to_science_log()
-            self.log().deactivate()
-            self.log().activate()
-            self.config().save()
-            self.review_controller().reset()
+            if self.config().server_only:
+                self.database().backup()
+                self.log().dump_to_science_log()
+            else:
+                self.flush_sync_server()
+                if not self.database() or not self.database().is_loaded() or \
+                    not self.database().is_accessible():
+                    # Make sure we don't continue if e.g. the GUI or another
+                    # thread holds the database.
+                    return
+                self.database().backup()
+                self.log().saved_database()
+                self.log().loaded_database()
+                self.log().future_schedule()
+                self.log().dump_to_science_log()
+                self.log().deactivate()
+                self.log().activate()
+                self.config().save()
+                self.reset_study_mode()
             self.next_rollover = self.database().start_of_day_n_days_ago(n=-1)
-        if time.time() > self.config()["last_db_maintenance"] + 90 * DAY:
-            self.component_manager.current("database_maintenance").run()        
+        if db_maintenance and \
+           (time.time() > self.config()["last_db_maintenance"] + 90 * DAY):
+            self.component_manager.current("database_maintenance").run()
+            self.config()["last_db_maintenance"] = time.time()
+            self.config().save()
+
+    def do_db_maintenance(self):
+        if time.time() < self.config()["last_db_maintenance"] + 30 * DAY:
+            self.main_widget().show_information(\
+          _("No need to do database maintenance more than once per month."))
+            return
+        else:
+            self.component_manager.current("database_maintenance").run()
             self.config()["last_db_maintenance"] = time.time()
             self.config().save()
 
@@ -80,11 +95,24 @@ class DefaultController(Controller):
             title += " - " + db.current_criterion().name
         self.main_widget().set_window_title(title)
 
+    def set_study_mode(self, study_mode):
+        if self.study_mode == study_mode:
+            return
+        if self.study_mode is not None:
+            self.study_mode.deactivate()
+        study_mode.activate()
+        self.study_mode = study_mode
+        self.config()["study_mode"] = study_mode.id
+
+    def reset_study_mode(self):
+        self.study_mode.deactivate()
+        self.study_mode.activate()
+
     def show_add_cards_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("add_cards_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         # This dialog calls 'create_new_cards' at some point.
         self.database().save()
         review_controller = self.review_controller()
@@ -95,6 +123,21 @@ class DefaultController(Controller):
             review_controller.update_status_bar_counters()
         self.stopwatch().unpause()
 
+    def _retain_only_child_tags(self, tag_names):
+
+        """In case e.g. tag_names is ["a", "a::b"], return only ["a::b"]."""
+
+        parent_tag_names = []
+        for tag_name in tag_names:
+            partial_tag = ""
+            for node in tag_name.split("::")[:-1]:
+                if partial_tag:
+                    partial_tag += "::"
+                partial_tag += node
+                if  partial_tag in tag_names:
+                    parent_tag_names.append(partial_tag)
+        return list(set(tag_names) - set(parent_tag_names))
+
     def create_new_cards(self, fact_data, card_type, grade, tag_names,
                          check_for_duplicates=True, save=True):
 
@@ -111,6 +154,7 @@ class DefaultController(Controller):
         assert grade in [-1, 2, 3, 4, 5] # Use -1 for yet to learn cards.
         assert card_type.is_fact_data_valid(fact_data)
         db = self.database()
+        tag_names = self._retain_only_child_tags(tag_names)
         tags = db.get_or_create_tags_with_names(tag_names)
         fact = Fact(fact_data)
         if check_for_duplicates:
@@ -175,8 +219,8 @@ class DefaultController(Controller):
                     card.fact.data = merged_fact_data
                     card.tags = tags
                     self.component_manager.current("edit_card_dialog")\
-                        (card, self.component_manager, allow_cancel=False).\
-                        activate()
+                        (card, component_manager=self.component_manager,
+                         allow_cancel=False).activate()
                     return db.cards_from_fact(fact)
         # Create cards.
         cards = card_type.create_sister_cards(fact)
@@ -191,17 +235,18 @@ class DefaultController(Controller):
         if save:
             db.save()
         if self.review_controller().learning_ahead == True:
-            self.review_controller().reset()
+            self.reset_study_mode()
         return cards
 
-    def show_edit_card_dialog(self): 
+    def show_edit_card_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         review_controller = self.review_controller()
         # This dialog calls 'edit_card_and_sisters' at some point.
         state = review_controller.state()
         accepted = self.component_manager.current("edit_card_dialog")\
-            (review_controller.card, self.component_manager).activate()
+            (review_controller.card,
+             component_manager=self.component_manager).activate()
         if not accepted:
             self.stopwatch().unpause()
             return
@@ -227,7 +272,7 @@ class DefaultController(Controller):
         """This is an internal function, used by 'edit_card_and_sisters' and
         'change_card_type'. It should not be called from the outside by
         itself, otherwise the database will not be saved.
-        
+
         Returns -2 for error, -1 for cancel and 0 for success.
 
         """
@@ -239,8 +284,8 @@ class DefaultController(Controller):
         assert cards_from_fact[0].card_type == old_card_type
         if not new_card_type.is_fact_data_valid(new_fact_data):
             self.main_widget().show_error(\
-                _("Card data not correctly formatted for conversion.\n\nSkipping ") +\
-                "|".join(fact.data.values()) + ".\n")
+        _("Card data not correctly formatted for conversion.\n\nSkipping ") +\
+                "|".join(list(fact.data.values())) + ".\n")
             return -2
         converter = self.component_manager.current\
               ("card_type_converter", used_for=(old_card_type.__class__,
@@ -319,7 +364,7 @@ class DefaultController(Controller):
             for card in edited_cards:
                 db.update_card(card)
             if new_cards and self.review_controller().learning_ahead:
-                self.review_controller().reset()
+                self.reset_study_mode()
             return 0
 
     def edit_card_and_sisters(self, card, new_fact_data, new_card_type,
@@ -333,7 +378,7 @@ class DefaultController(Controller):
         fact = db.fact(card.fact._id, is_id_internal=True)
         current_sister_cards = self.database().cards_from_fact(fact)
         current_tag_strings = set([sister_card.tag_string() \
-            for sister_card in current_sister_cards])        
+            for sister_card in current_sister_cards])
         # Change the card type if needed. This does not take into account
         # changes to fact yet, which will come just afterwards.
         result = self._change_card_type(card.fact, card.card_type,
@@ -359,7 +404,7 @@ class DefaultController(Controller):
         for edited_card in edited_cards:
             db.update_card(edited_card)
         if new_cards and self.review_controller().learning_ahead == True:
-            self.review_controller().reset()
+            self.reset_study_mode()
         # Apply new tags and modification time to cards and save them back to
         # the database. Note that this makes sure there is an EDITED_CARD log
         # entry for each sister card, which is needed when syncing with a
@@ -367,9 +412,10 @@ class DefaultController(Controller):
         tag_for_current_card_only = False
         if len(current_tag_strings) > 1:
             tag_for_current_card_only = bool(self.main_widget().show_question(
-            _("This card has different tags than its sister cards. Update tags for current card only or for all sister cards?"),
+_("This card has different tags than its sister cards. Update tags for current card only or for all sister cards?"),
             _("Current card only"), _("All sister cards"), "") == 0)
         old_tags = set()
+        new_tag_names = self._retain_only_child_tags(new_tag_names)
         tags = db.get_or_create_tags_with_names(new_tag_names)
         modification_time = int(time.time())
         for sister_card in self.database().cards_from_fact(fact):
@@ -397,20 +443,20 @@ class DefaultController(Controller):
         for fact in facts:
             if correspondence:
                 new_fact_data = {}
-                for old_fact_key, new_fact_key in correspondence.iteritems():
+                for old_fact_key, new_fact_key in correspondence.items():
                     if old_fact_key in fact:
                         new_fact_data[new_fact_key] = fact[old_fact_key]
                 if new_card_type.is_fact_data_valid(new_fact_data):
                     fact.data = new_fact_data
             else:
-                new_fact_data = copy.copy(fact.data)  
+                new_fact_data = copy.copy(fact.data)
             result = self._change_card_type(fact, old_card_type,
                     new_card_type, correspondence, new_fact_data, warn)
             if result == -1:  # Cancel.
                 w.close_progress()
-                return                    
-            if correspondence and result != -2:  # Error. 
-                db.update_fact(fact)              
+                return
+            if correspondence and result != -2:  # Error.
+                db.update_fact(fact)
             warn = False
             w.increase_progress(1)
         db.save()
@@ -422,7 +468,7 @@ class DefaultController(Controller):
         if self.config()["star_help_shown"] == False:
             self.main_widget().show_information(\
 _("This will add a tag 'Starred' to the current card, so that you can find it back easily, e.g. to edit it on a desktop."))
-            self.config()["star_help_shown"] = True        
+            self.config()["star_help_shown"] = True
         db = self.database()
         review_controller = self.review_controller()
         tag = db.get_or_create_tag_with_name(_("Starred"))
@@ -575,7 +621,7 @@ _("It is recommended to put all your cards in a single database. Using tags to d
         self.main_widget().close_progress()
         db.load(self.config()["last_database"])
         self.log().loaded_database()
-        self.review_controller().reset()
+        self.reset_study_mode()
         self.review_controller().update_dialog()
         self.update_title()
         self.stopwatch().unpause()
@@ -611,7 +657,7 @@ _("'*.cards' files are not separate databases, but need to be imported in your c
                 # case, as the user wants to throw it away. This mainly
                 # prohibits dumping to the science log.
                 db.restore(filename)
-                self.review_controller().reset()
+                self.reset_study_mode()
                 self.update_title()
             self.stopwatch().unpause()
             return
@@ -620,21 +666,21 @@ _("'*.cards' files are not separate databases, but need to be imported in your c
                 self.log().saved_database()
                 db.backup()
                 db.unload()
-            except RuntimeError, error:
-                self.main_widget().show_error(unicode(error.message))
+            except RuntimeError as error:
+                self.main_widget().show_error(str(error))
                 self.stopwatch().unpause()
                 return
         try:
             db.load(filename)
             self.log().loaded_database()
             self.log().future_schedule()
-        except Exception, error:
-            self.main_widget().show_error(unicode(error.message))
+        except Exception as error:
+            self.main_widget().show_error(str(error))
             db.abandon()
             db.load(old_path)
             self.stopwatch().unpause()
             return
-        self.review_controller().reset()
+        self.reset_study_mode()
         self.update_title()
         self.stopwatch().unpause()
 
@@ -649,8 +695,8 @@ _("Your database will be autosaved before exiting. Also, it is saved every coupl
             self.database().save()
             self.log().saved_database()
             self.main_widget().show_information(_("Database saved."))
-        except RuntimeError, error:
-            self.main_widget().show_error(unicode(error.message))
+        except RuntimeError as error:
+            self.main_widget().show_error(str(error))
         self.stopwatch().unpause()
 
     def show_save_file_as_dialog(self):
@@ -671,7 +717,7 @@ _("Your database will be autosaved before exiting. Also, it is saved every coupl
             self.main_widget().show_information(\
 _("The configuration database cannot be used to store cards."))
             self.stopwatch().unpause()
-            return          
+            return
         if not filename.endswith(suffix):
             filename += suffix
         try:
@@ -679,12 +725,13 @@ _("The configuration database cannot be used to store cards."))
             new_media_dir = self.database().media_dir()
             if old_media_dir == new_media_dir:
                 return
+            import shutil
             if os.path.exists(new_media_dir):
                 shutil.rmtree(new_media_dir)
             shutil.copytree(old_media_dir, new_media_dir)
             self.log().saved_database()
-        except RuntimeError, error:
-            self.main_widget().show_error(unicode(error.message))
+        except RuntimeError as error:
+            self.main_widget().show_error(str(error))
             self.stopwatch().unpause()
             return
         self.review_controller().update_dialog()
@@ -695,10 +742,10 @@ _("The configuration database cannot be used to store cards."))
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("compact_database_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.review_controller().reset_but_try_to_keep_current_card()
-        self.stopwatch().unpause()       
-            
+        self.stopwatch().unpause()
+
     def show_insert_img_dialog(self, filter):
 
         """Show a file dialog filtered on the supported filetypes, get a
@@ -776,7 +823,7 @@ _("The configuration database cannot be used to store cards."))
         self.flush_sync_server()
         review_controller = self.review_controller()
         self.component_manager.current("browse_cards_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         review_controller.reset_but_try_to_keep_current_card()
         review_controller.update_dialog(redraw_all=True)
         self.stopwatch().unpause()
@@ -784,14 +831,14 @@ _("The configuration database cannot be used to store cards."))
     def show_activate_cards_dialog(self):
         self.show_activate_cards_dialog_pre()
         self.show_activate_cards_dialog_post()
-         
+
     def show_activate_cards_dialog_pre(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("activate_cards_dialog")\
-            (self.component_manager).activate()
-        
-    def show_activate_cards_dialog_post(self):       
+            (component_manager=self.component_manager).activate()
+
+    def show_activate_cards_dialog_post(self):
         review_controller = self.review_controller()
         review_controller.reset_but_try_to_keep_current_card()
         review_controller.update_status_bar_counters()
@@ -815,7 +862,7 @@ _("This will tag all the cards in a given card type which have the same question
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("manage_plugins_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.review_controller().update_dialog(redraw_all=True)
         self.stopwatch().unpause()
 
@@ -840,7 +887,7 @@ _("This will tag all the cards in a given card type which have the same question
         plugin_filename, plugin_class_name = None, None
         for filename in filenames:
             if filename.endswith(".py"):
-                text = file(os.path.join(plugin_dir, filename), "r").read()
+                text = open(os.path.join(plugin_dir, filename), "r").read()
                 match = re_plugin.match(text)
                 if match is not None:
                     plugin_filename = filename
@@ -850,11 +897,11 @@ _("This will tag all the cards in a given card type which have the same question
             self.main_widget().show_error(_("No plugin found!"))
             return
         # Write manifest to allow uninstalling.
-        manifest = file(os.path.join(plugin_dir,
+        manifest = open(os.path.join(plugin_dir,
             plugin_class_name + ".manifest"), "w")
         for filename in filenames:
             if not os.path.isdir(os.path.join(plugin_dir, filename)):
-                print >> manifest, filename
+                print(filename, file=manifest)
         # Make sure we don't register a plugin twice.
         for plugin in self.plugins():
             if plugin.__class__.__name__ == plugin_class_name:
@@ -867,7 +914,8 @@ _("This will tag all the cards in a given card type which have the same question
             if module_name in sys.modules:
                 del sys.modules[module_name]
             __import__(module_name)
-        except:
+        except Exception as e:
+            print(e)
             from mnemosyne.libmnemosyne.utils import traceback_string
             msg = _("Error when running plugin:") \
                 + "\n" + traceback_string()
@@ -882,7 +930,7 @@ _("This will tag all the cards in a given card type which have the same question
         plugin_dir = os.path.join(self.config().data_dir, "plugins")
         manifest_filename = os.path.join(plugin_dir,
             plugin.__class__.__name__ + ".manifest")
-        manifest = file(manifest_filename, "r")
+        manifest = open(manifest_filename, "r")
         plugin_dir = os.path.join(self.config().data_dir, "plugins")
         for filename in manifest:
             filename = os.path.join(plugin_dir, filename.rstrip())
@@ -898,21 +946,21 @@ _("This will tag all the cards in a given card type which have the same question
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("manage_card_types_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
 
     def show_statistics_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("statistics_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
 
     def show_configuration_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("configuration_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.config().save()
         self.review_controller().reset_but_try_to_keep_current_card()
         self.review_controller().update_dialog(redraw_all=True)
@@ -922,7 +970,7 @@ _("This will tag all the cards in a given card type which have the same question
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("import_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.database().save()
         self.log().saved_database()
         review_controller = self.review_controller()
@@ -936,33 +984,20 @@ _("This will tag all the cards in a given card type which have the same question
         self.stopwatch().pause()
         self.flush_sync_server()
         self.component_manager.current("export_dialog")\
-            (self.component_manager).activate()
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
 
-    def show_export_metadata_dialog(self, metadata=None, read_only=False):
-        self.stopwatch().pause()
-        self.flush_sync_server()
-        dialog = self.component_manager.current("export_metadata_dialog")\
-            (self.component_manager)
-        if metadata:
-            dialog.set_values(metadata)
-        if read_only:
-            dialog.set_read_only()
-        dialog.activate()
-        self.stopwatch().unpause()
-        return dialog.values()
-
     def show_sync_dialog(self):
         self.show_sync_dialog_pre()
         self.show_sync_dialog_post()
-        
-    def show_sync_dialog_pre(self): 
+
+    def show_sync_dialog_pre(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         self.database().save()
         self.component_manager.current("sync_dialog")\
-            (self.component_manager).activate()
-        
+        (component_manager=self.component_manager).activate()
+
     def show_sync_dialog_post(self):
         self.database().save()
         self.log().saved_database()
@@ -987,7 +1022,7 @@ _("This will tag all the cards in a given card type which have the same question
             self.database().store_pregenerated_data
         client.do_backup = self.config()["backup_before_sync"]
         client.upload_science_logs = self.config()["upload_science_logs"]
-        try:         
+        try:
             client.sync(server, port, username, password)
         finally:
             client.database.release_connection()
@@ -1021,20 +1056,20 @@ _("This will tag all the cards in a given card type which have the same question
     def show_getting_started_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
-        self.component_manager.current("getting_started_dialog")(
-           self.component_manager).activate()
+        self.component_manager.current("getting_started_dialog")\
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
 
     def show_tip_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
-        self.component_manager.current("tip_dialog")(
-            self.component_manager).activate()
+        self.component_manager.current("tip_dialog")\
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
 
     def show_about_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
-        self.component_manager.current("about_dialog")(
-            self.component_manager).activate()
+        self.component_manager.current("about_dialog")\
+            (component_manager=self.component_manager).activate()
         self.stopwatch().unpause()
