@@ -18,8 +18,9 @@ from mnemosyne.libmnemosyne.card import Card
 from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.libmnemosyne.card_type import CardType
 from mnemosyne.libmnemosyne.fact_view import FactView
-from mnemosyne.libmnemosyne.utils import path_exists, path_join
-from mnemosyne.libmnemosyne.utils import expand_path, MnemosyneError
+from mnemosyne.libmnemosyne.utils import MnemosyneError
+from mnemosyne.libmnemosyne.utils import normalise_path, expand_path
+
 
 re_src = re.compile(r"""(src|data)=\"(.+?)\"""", re.DOTALL | re.IGNORECASE)
 
@@ -119,7 +120,7 @@ class SQLiteSync(object):
         # At the end of the sync, see if we were able to actually instantiate
         # all card types.
         if len(self.card_types_to_instantiate_later) != 0:
-            raise RuntimeError, _("Missing plugins for card types.")
+            raise RuntimeError(_("Missing plugins for card types."))
         # See if we need to reapply the default criterion.
         if self.reapply_default_criterion_needed:
             criterion = self.current_criterion()
@@ -186,8 +187,9 @@ class SQLiteSync(object):
             """select object_id from log where _id>? and (event_type=? or
             event_type=?)""", (_id, EventTypes.ADDED_MEDIA_FILE,
             EventTypes.EDITED_MEDIA_FILE))]:
-            if path_exists(expand_path(filename, self.media_dir())):
-                filenames.add(filename)                
+            if os.path.exists(\
+                normalise_path(expand_path(filename, self.media_dir()))):
+                filenames.add(filename)
         return filenames
 
     def all_media_filenames(self):
@@ -197,13 +199,12 @@ class SQLiteSync(object):
         # We cannot rely on logs in the database here, since part of it
         # may have been archived, so we simply send across the entire
         # media directory.
-        
+
         filenames = set()
-        # 'str' needed for Android:
-        for root, dirs, files in os.walk(str(self.media_dir())):
+        for root, dirs, files in os.walk(self.media_dir()):
             subdir = root.replace(self.media_dir(), "")
             for name in files:
-                filename = path_join(subdir, name)
+                filename = os.path.join(subdir, name)
                 if filename.startswith("\\") or filename.startswith("/"):
                     filename = filename[1:]
                 filenames.add(filename)
@@ -269,7 +270,7 @@ class SQLiteSync(object):
             _fact_id from cards where active=1) and value like '%src=%'"""):
             for match in re_src.finditer(result[0]):
                 active_objects["media_filenames"].add(match.group(2))
-        return active_objects     
+        return active_objects
 
     def set_extra_tags_on_import(self, tags):
         self.extra_tags_on_import = tags
@@ -357,7 +358,7 @@ class SQLiteSync(object):
                 return None
             if self.has_fact_with_id(log_entry["o_id"]):
                 fact = self.fact(log_entry["o_id"], is_id_internal=False)
-                for fact_key, value in fact.data.iteritems():
+                for fact_key, value in fact.data.items():
                     log_entry[fact_key] = value
             else: # The object has been deleted at a later stage.
                 pass
@@ -412,10 +413,10 @@ class SQLiteSync(object):
     def add_tag_from_log_entry(self, log_entry):
         already_imported = self.has_tag_with_id(log_entry["o_id"])
         if "name" not in log_entry:
-            log_entry["name"] = "dummy"  # Added and immediately deleted.        
+            log_entry["name"] = "dummy"  # Added and immediately deleted.
         same_name_in_database = self.con.execute(\
-            "select count() from tags where name=? and id!=?",
-            (log_entry["name"], log_entry["o_id"])).fetchone()[0] == 1
+            "select 1 from tags where name=? and id!=? limit 1",
+            (log_entry["name"], log_entry["o_id"])).fetchone() is not None
         if same_name_in_database:
             # Merging with the tag which is already in the database is more
             # difficult, as then the tag links in the cards would need to
@@ -435,7 +436,7 @@ class SQLiteSync(object):
         # the object from the database. This is a bit slower than just filling
         # in harmless missing keys, but it is more robust against future
         # side effects of tag deletion.
-        if log_entry["type"] == EventTypes.DELETED_TAG:           
+        if log_entry["type"] == EventTypes.DELETED_TAG:
             # Work around legacy logs which contain duplicate deletion events.
             if self.has_tag_with_id(log_entry["o_id"]):
                 return self.tag(log_entry["o_id"], is_id_internal=False)
@@ -444,7 +445,7 @@ class SQLiteSync(object):
                 self.main_widget().show_information(\
             _("Deleting same tag twice during sync. Inform the developpers."))
                 log_entry["name"] = "irrelevant"
-                return Tag(log_entry["name"], log_entry["o_id"])                
+                return Tag(log_entry["name"], log_entry["o_id"])
         # If we are creating a tag that will be deleted at a later stage
         # during this sync, we are missing some (irrelevant) information
         # needed to properly create a tag object.
@@ -476,22 +477,22 @@ class SQLiteSync(object):
         if log_entry["type"] != EventTypes.ADDED_FACT and \
            not self.has_fact_with_id(log_entry["o_id"]):
             self.main_widget().show_information(\
-        _("Deleting same fact twice during sync. Inform the developpers."))                
+        _("Deleting same fact twice during sync. Inform the developpers."))
             fact = Fact({}, log_entry["o_id"])
             fact._id = -1
-            return fact             
+            return fact
         # Get fact object to be deleted now.
         if log_entry["type"] == EventTypes.DELETED_FACT:
             return self.fact(log_entry["o_id"], is_id_internal=False)
         # Create fact object.
         fact_data = {}
-        for key, value in log_entry.iteritems():
+        for key, value in log_entry.items():
             if key not in ["time", "type", "o_id"]:
                 fact_data[key] = value
         fact = Fact(fact_data, log_entry["o_id"])
         if log_entry["type"] != EventTypes.ADDED_FACT:
             fact._id = self.con.execute("select _id from facts where id=?",
-                (fact.id, )).fetchone()[0]                
+                (fact.id, )).fetchone()[0]
         return fact
 
     def add_card_from_log_entry(self, log_entry):
@@ -541,7 +542,7 @@ class SQLiteSync(object):
                 return self.card(log_entry["o_id"], is_id_internal=False)
             except MnemosyneError:  # There is no fact in the database.
                 # We have created and deleted this card since the last sync,
-                # so we just return an empty shell.                              
+                # so we just return an empty shell.
                 card_type = self.card_type_with_id("1")
                 fact = Fact({"f": "f", "b": "b"}, id="")
                 card = Card(card_type, fact, card_type.fact_views[0],
@@ -566,10 +567,10 @@ class SQLiteSync(object):
             if log_entry["card_t"] not in \
                 self.component_manager.card_type_with_id:
                 # If the card type is not in the database, it's possible that
-                # the data for this card type will follow later during the 
-                # sync. In that case, create a dummy card type here, which 
-                # will be corrected by a later edit event. Hovewer, we still 
-                # need to instantiate this card type later, so that we can 
+                # the data for this card type will follow later during the
+                # sync. In that case, create a dummy card type here, which
+                # will be corrected by a later edit event. Hovewer, we still
+                # need to instantiate this card type later, so that we can
                 # catch errors, e.g. due to bad plugins.
                 try:
                     self.activate_plugins_for_card_type_with_id\
@@ -666,8 +667,8 @@ class SQLiteSync(object):
         """
 
         filename = log_entry["fname"]
-        full_path = expand_path(filename, self.media_dir())
-        if path_exists(full_path):
+        full_path = normalise_path(expand_path(filename, self.media_dir()))
+        if os.path.exists(full_path):
             self.con.execute("""insert or replace into media(filename, _hash)
                 values(?,?)""", (filename, self._media_hash(filename)))
         self.log().added_media_file(filename)
@@ -680,23 +681,23 @@ class SQLiteSync(object):
 
     def delete_media_file(self, log_entry):
         # Actually, we cannot take the responsibility to delete a media file,
-        # since it would e.g. break the corner case to add a media file, 
+        # since it would e.g. break the corner case to add a media file,
         # delete it and than add it again.
         pass
-    
+
         #filename = log_entry["fname"]
         #full_path = expand_path(filename, self.media_dir())
         # The file could have been remotely deleted before it got a chance to
         # be synced, so we need to check if the file exists before deleting.
-        #if path_exists(full_path):
+        #if os.path_exists(full_path):
         #    os.remove(full_path)
         #self.log().deleted_media_file(filename)
 
     def add_fact_view_from_log_entry(self, log_entry):
         if self.importing:
             already_imported = self.con.execute(\
-                "select count() from fact_views where id=?",
-                (log_entry["o_id"], )).fetchone()[0] != 0
+                "select 1 from fact_views where id=? limit 1",
+                (log_entry["o_id"], )).fetchone() is not None
             # No need to rename fact views here, as the user only names the
             # card types.
             if already_imported:
@@ -732,13 +733,13 @@ class SQLiteSync(object):
 
     def add_card_type_from_log_entry(self, log_entry):
         already_imported = self.con.execute(\
-            "select count() from card_types where id=?",
-            (log_entry["o_id"], )).fetchone()[0] != 0
+            "select 1 from card_types where id=? limit 1",
+            (log_entry["o_id"], )).fetchone() is not None
         if "name" not in log_entry:
             log_entry["name"] = "dummy"  # Added and immediately deleted.
         same_name_in_database = self.con.execute(\
-            "select count() from card_types where name=? and id!=?",
-            (log_entry["name"], log_entry["o_id"] )).fetchone()[0] == 1
+            "select 1 from card_types where name=? and id!=? limit 1",
+            (log_entry["name"], log_entry["o_id"] )).fetchone() is not None
         if same_name_in_database:
             # Merging with the card type which is already in the database
             # is more difficult, as then the card type links in the cards
@@ -759,7 +760,7 @@ class SQLiteSync(object):
         except sqlite3.IntegrityError:
             # Leftover from old bug, should not reoccur.
             self.main_widget().show_information(\
-    _("Creating same card type twice during sync. Inform the developpers."))            
+    _("Creating same card type twice during sync. Inform the developpers."))
 
     def card_type_from_log_entry(self, log_entry):
         # Get card type object to be deleted now.
@@ -786,7 +787,7 @@ class SQLiteSync(object):
         card_type.required_fact_keys = eval(log_entry["required_fact_keys"])
         card_type.keyboard_shortcuts = eval(log_entry["keyboard_shortcuts"])
         if "extra" in log_entry:
-            card_type.extra_data = eval(log_entry["extra"])           
+            card_type.extra_data = eval(log_entry["extra"])
         return card_type
 
     def criterion_from_log_entry(self, log_entry):
@@ -825,11 +826,11 @@ class SQLiteSync(object):
             self.importing = True
         event_type = log_entry["type"]
         if "time" in log_entry:
-            self.log().timestamp = int(log_entry["time"]) 
+            self.log().timestamp = int(log_entry["time"])
         # TMP measure to allow syncing partners which did not yet store
         # machine ids for LOADED_DATABASE and SAVED_DATABASE.
         if not "o_id" in log_entry:
-            log_entry["o_id"] = ""        
+            log_entry["o_id"] = ""
         try:
             if event_type == EventTypes.STARTED_PROGRAM:
                 self.log().started_program(log_entry["o_id"])
@@ -891,7 +892,7 @@ class SQLiteSync(object):
                     self.reapply_default_criterion_needed = True
             elif event_type == EventTypes.DELETED_CRITERION:
                 self.delete_criterion(self.criterion_from_log_entry(log_entry))
-            elif event_type == EventTypes.EDITED_SETTING:              
+            elif event_type == EventTypes.EDITED_SETTING:
                 key, value = log_entry["o_id"], eval(log_entry["value"])
                 if key in self.config().keys_to_sync:
                     self.config()[key] = value
