@@ -4,7 +4,7 @@
 
 import copy
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 
 from mnemosyne.libmnemosyne.fact import Fact
 from mnemosyne.libmnemosyne.translator import _
@@ -33,9 +33,10 @@ class AddEditCards(TipAfterStartingNTimes):
          18: _("If you use 'Edit cards', changes are made to all the sister cards simultaneously.")}
 
     def activate(self):
+        AddCardsDialog.activate(self)
         self.show_tip_after_starting_n_times()
         status = self.exec_()
-        return (status == QtGui.QDialog.Accepted)
+        return (status == QtWidgets.QDialog.Accepted)
 
     def initialise_card_types_combobox(self, current_card_type_name):
         # We calculate card_type_by_name here because these names can change
@@ -48,6 +49,12 @@ class AddEditCards(TipAfterStartingNTimes):
         self.previous_card_type_name = current_card_type_name
         db_sorted_card_types = self.database().sorted_card_types()
         for card_type in db_sorted_card_types:
+            if card_type.hidden_from_UI == True:
+                continue
+            # Adding M-sided cards or converting to them is not (yet) supported.
+            if _(card_type.name) != current_card_type_name \
+               and card_type.id.startswith("7"):
+                continue
             if _(card_type.name) == current_card_type_name:
                 self.card_type = card_type
                 self.card_type_index = self.card_types_widget.count()
@@ -58,7 +65,7 @@ class AddEditCards(TipAfterStartingNTimes):
             self.card_type_index = 0
         self.card_types_widget.setCurrentIndex(self.card_type_index)
         # Now that the combobox is filled, we can connect the signal.
-        self.card_types_widget.currentIndexChanged[QtCore.QString].\
+        self.card_types_widget.currentIndexChanged[str].\
             connect(self.card_type_changed)
         self.correspondence = {}  # Used when changing card types.
         self.update_card_widget()
@@ -92,17 +99,18 @@ class AddEditCards(TipAfterStartingNTimes):
         if not keep_data_from_previous_widget:
             prefill_fact_data = self.card.fact.data
         # Show new card type widget.
-        card_type_name = unicode(self.card_types_widget.currentText())
+        card_type_name = self.card_types_widget.currentText()
         self.card_type = self.card_type_by_name[card_type_name]
         try:
             self.card_type_widget = self.component_manager.current \
                 ("card_type_widget", used_for=self.card_type.__class__) \
-                (self.component_manager, parent=self)
-        except:
+                (card_type=self.card_type, parent=self,
+                 component_manager=self.component_manager)
+        except Exception as e:
             if not self.card_type_widget:
                 self.card_type_widget = self.component_manager.current \
-                    ("generic_card_type_widget")(self.component_manager,
-                    parent=self, card_type=self.card_type)
+                    ("generic_card_type_widget")(card_type=self.card_type,
+                    parent=self, component_manager=self.component_manager)
         self.card_type_widget.set_fact_data(prefill_fact_data)
         self.card_type_widget.show()
         self.vbox_layout.insertWidget(1, self.card_type_widget)
@@ -118,7 +126,7 @@ class AddEditCards(TipAfterStartingNTimes):
                 existing_current_tag_names.append(tag.name)
         current_tag_name = ", ".join(existing_current_tag_names)
         # For the 'special' tags, we add them at the top.
-        self.tags.setInsertPolicy(QtGui.QComboBox.InsertAtTop)
+        self.tags.setInsertPolicy(QtWidgets.QComboBox.InsertAtTop)
         if "," in current_tag_name:
             self.tags.addItem(current_tag_name)
         if current_tag_name == "":
@@ -127,10 +135,11 @@ class AddEditCards(TipAfterStartingNTimes):
             if self.tags.itemText(i) == current_tag_name:
                 self.tags.setCurrentIndex(i)
                 break
-        self.previous_tags = unicode(self.tags.currentText())
+        self.previous_tags = self.tags.currentText()
+        self.tags.refresh_completion_model()
 
     def card_type_changed(self, new_card_type_name):
-        new_card_type_name = unicode(new_card_type_name)
+        new_card_type_name = new_card_type_name
         new_card_type = self.card_type_by_name[new_card_type_name]
         if self.card_type.fact_keys().issubset(new_card_type.fact_keys()) or \
             self.card_type_widget.is_empty():
@@ -138,7 +147,7 @@ class AddEditCards(TipAfterStartingNTimes):
             return
         dlg = ConvertCardTypeKeysDlg(self.card_type, new_card_type,
             self.correspondence, check_required_fact_keys=False, parent=self)
-        if dlg.exec_() != QtGui.QDialog.Accepted:
+        if dlg.exec_() != QtWidgets.QDialog.Accepted:
             # Set correspondence so as not to erase previous data.
             self.correspondence = {}
             for key in self.card_type.fact_keys():
@@ -152,8 +161,9 @@ class AddEditCards(TipAfterStartingNTimes):
         fact_data = self.card_type_widget.fact_data()
         fact = Fact(fact_data)
         cards = self.card_type.create_sister_cards(fact)
-        tag_text = unicode(self.tags.currentText())
-        dlg = PreviewCardsDlg(self.component_manager, cards, tag_text, self)
+        tag_text = self.tags.currentText()
+        dlg = PreviewCardsDlg(cards, tag_text,
+            component_manager=self.component_manager, parent=self)
         dlg.exec_()
 
     def __del__(self):
@@ -161,22 +171,26 @@ class AddEditCards(TipAfterStartingNTimes):
         self.card_type_widget = None
 
 
-class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
+class AddCardsDlg(QtWidgets.QDialog, AddEditCards,
+                  AddCardsDialog, Ui_AddCardsDlg):
 
-    def __init__(self, component_manager):
-        AddEditCards.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+    def __init__(self, card_type=None, fact_data=None, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         self.setWindowFlags(self.windowFlags() \
             | QtCore.Qt.WindowMinMaxButtonsHint)
         self.setWindowFlags(self.windowFlags() \
             & ~ QtCore.Qt.WindowContextHelpButtonHint)
-        last_used_card_type_id = self.config()["last_used_card_type_id"]
-        try:
-            last_used_card_type = self.card_type_with_id(last_used_card_type_id)
-        except:
-            # First time use, or card type was deleted.
-            last_used_card_type = self.card_type_with_id("1")
+        if card_type:
+            last_used_card_type = card_type
+        else:
+            last_used_card_type_id = self.config()["last_used_card_type_id"]
+            try:
+                last_used_card_type = self.card_type_with_id(\
+                    last_used_card_type_id)
+            except:
+                # First time use, or card type was deleted.
+                last_used_card_type = self.card_type_with_id("1")
         self.initialise_card_types_combobox(last_used_card_type.name)
         if last_used_card_type.id not in \
             self.config()["last_used_tags_for_card_type_id"]:
@@ -187,7 +201,9 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
         else:
             self.update_tags_combobox(self.config()\
                 ["last_used_tags_for_card_type_id"][last_used_card_type.id])
-        self.grades = QtGui.QButtonGroup()
+        if fact_data:
+            self.card_type_widget.set_fact_data(fact_data)
+        self.grades = QtWidgets.QButtonGroup()
         # Negative indexes have special meanings in Qt, so we can't use -1 for
         # 'yet to learn'.
         self.grades.addButton(self.yet_to_learn_button, 0)
@@ -196,29 +212,29 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
         self.grades.addButton(self.grade_4_button, 4)
         self.grades.addButton(self.grade_5_button, 5)
         self.grades.buttonClicked[int].connect(self.create_new_cards)
-        self.set_valid(False)
+        if not (card_type and fact_data):
+            self.set_valid(False)
         state = self.config()["add_cards_dlg_state"]
         if state:
             self.restoreGeometry(state)
-            
+
     def card_type_changed(self, new_card_type_name):
         # We only store the last used tags when creating a new card,
         # not when editing.
-        new_card_type_name = unicode(new_card_type_name)
-        new_card_type = self.card_type_by_name[new_card_type_name] 
+        new_card_type = self.card_type_by_name[new_card_type_name]
         self.config()["last_used_card_type_id"] = new_card_type.id
         if new_card_type.id not in \
             self.config()["last_used_tags_for_card_type_id"]:
             self.config()["last_used_tags_for_card_type_id"]\
                 [new_card_type.id] = ""
         if not self.config()["is_last_used_tags_per_card_type"]:
-            if not unicode(self.tags.currentText()):
+            if not self.tags.currentText():
                 self.update_tags_combobox(self.config()["last_used_tags"])
         else:
-            if not unicode(self.tags.currentText()) \
+            if not self.tags.currentText() \
                 or self.card_type_widget.is_empty():
                 self.update_tags_combobox(self.config()\
-                    ["last_used_tags_for_card_type_id"][new_card_type.id]) 
+                    ["last_used_tags_for_card_type_id"][new_card_type.id])
         AddEditCards.card_type_changed(self, new_card_type_name)
 
     def keyPressEvent(self, event):
@@ -241,7 +257,7 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
             elif event.key() == QtCore.Qt.Key_E:
                 self.reject()
         else:
-            QtGui.QDialog.keyPressEvent(self, event)
+            QtWidgets.QDialog.keyPressEvent(self, event)
 
     def set_valid(self, valid):
         self.grade_buttons.setEnabled(valid)
@@ -252,8 +268,8 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
             grade = -1
         fact_data = self.card_type_widget.fact_data()
         tag_names = [c.strip() for c in \
-                     unicode(self.tags.currentText()).split(',')]
-        card_type_name = unicode(self.card_types_widget.currentText())
+                     self.tags.currentText().split(',')]
+        card_type_name = self.card_types_widget.currentText()
         card_type = self.card_type_by_name[card_type_name]
         c = self.controller()
         c.create_new_cards(fact_data, card_type, grade, tag_names, save=True)
@@ -265,15 +281,17 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
         self.card_type_widget.clear()
 
     def reject(self):
+        # Generated when pressing escape or clicking the exit button.
+        self._store_state()
         if not self.card_type_widget.is_empty() and \
             self.card_type_widget.is_changed():
             status = self.main_widget().show_question(\
                 _("Abandon current card?"), _("&Yes"), _("&No"), "")
             if status == 0:
-                QtGui.QDialog.reject(self)
+                QtWidgets.QDialog.reject(self)
                 return
         else:
-            QtGui.QDialog.reject(self)
+            QtWidgets.QDialog.reject(self)
 
     def _store_state(self):
         self.config()["add_cards_dlg_state"] = self.saveGeometry()
@@ -287,4 +305,4 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
     def accept(self):
         # 'accept' does not generate a close event.
         self._store_state()
-        return QtGui.QDialog.accept(self)
+        return QtWidgets.QDialog.accept(self)
