@@ -2,7 +2,7 @@
 # criterion_wdgt_default.py <Peter.Bienstman@UGent.be>
 #
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 
 from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.libmnemosyne.ui_components.criterion_widget \
@@ -13,8 +13,8 @@ from mnemosyne.pyqt_ui.card_type_tree_wdgt import CardTypesTreeWdgt
 from mnemosyne.pyqt_ui.ui_criterion_wdgt_default import Ui_DefaultCriterionWdgt
 
 
-class DefaultCriterionWdgt(QtGui.QWidget, Ui_DefaultCriterionWdgt,
-                           CriterionWidget):
+class DefaultCriterionWdgt(QtWidgets.QWidget, CriterionWidget,
+                           Ui_DefaultCriterionWdgt):
 
     """Note that this dialog can support active tags and forbidden tags,
     but not at the same time, in order to keep the interface compact.
@@ -23,19 +23,21 @@ class DefaultCriterionWdgt(QtGui.QWidget, Ui_DefaultCriterionWdgt,
 
     used_for = DefaultCriterion
 
-    def __init__(self, component_manager, parent):
-        CriterionWidget.__init__(self, component_manager)
-        QtGui.QWidget.__init__(self, parent)
-        self.parent = parent
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
+        self.parent = kwds["parent"]
+        component_manager = kwds["component_manager"]
         self.setupUi(self)
-        self.card_type_tree_wdgt = CardTypesTreeWdgt(component_manager, self)
+        self.card_type_tree_wdgt = CardTypesTreeWdgt(\
+            component_manager=component_manager, parent=self)
         # Bug in Qt: need to explicitly reset the text of this label.
         self.label.setText(_("Activate cards from these card types:"))
         self.gridLayout.addWidget(self.card_type_tree_wdgt, 1, 0)
-        self.tag_tree_wdgt = TagsTreeWdgt(component_manager, self)
+        self.tag_tree_wdgt = TagsTreeWdgt(\
+            component_manager=component_manager, parent=self)
         self.gridLayout.addWidget(self.tag_tree_wdgt, 1, 1)
-        
-        criterion = DefaultCriterion(self.component_manager)
+
+        criterion = DefaultCriterion(component_manager=self.component_manager)
         for tag in self.database().tags():
             criterion._tag_ids_active.add(tag._id)
         self.display_criterion(criterion)
@@ -63,7 +65,7 @@ class DefaultCriterionWdgt(QtGui.QWidget, Ui_DefaultCriterionWdgt,
 
         """
 
-        criterion = DefaultCriterion(self.component_manager)
+        criterion = DefaultCriterion(component_manager=self.component_manager)
         criterion = self.card_type_tree_wdgt.checked_to_criterion(criterion)
         # Tag tree contains active tags.
         if self.active_or_forbidden.currentIndex() == 0:
@@ -75,15 +77,15 @@ class DefaultCriterionWdgt(QtGui.QWidget, Ui_DefaultCriterionWdgt,
             for tag in self.database().tags():
                 criterion._tag_ids_active.add(tag._id)
         return criterion
-    
+
     def criterion_clicked(self):
         if self.parent.was_showing_a_saved_set and not self.parent.is_shutting_down:
             self.main_widget().show_information(\
-                _("Cards you (de)activate now will not be stored in the previously selected set unless you click 'Save this set for later use' again. This allows you to make some quick-and-dirty modifications."))
+_("Cards you (de)activate now will not be stored in the previously selected set unless you click 'Save this set for later use' again. This allows you to make some quick-and-dirty modifications."))
             self.parent.was_showing_a_saved_set = False
 
     def criterion_changed(self):
-        self.parent.saved_sets.clearSelection()       
+        self.parent.saved_sets.clearSelection()
 
     def closeEvent(self, event):
         # This allows the state of the tag tree to be saved.
