@@ -2,19 +2,20 @@
 # main_wdgt.py <Peter.Bienstman@UGent.be>
 #
 
-import sys
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 
 from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.pyqt_ui.ui_main_wdgt import Ui_MainWdgt
 from mnemosyne.libmnemosyne.ui_components.main_widget import MainWidget
 
+# Note: inheritance from Ui_MainWdgt should come last, as it inherits
+# directly from 'object' with supporting a correct super().__init__
+# call.
 
-class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
+class MainWdgt(QtWidgets.QMainWindow, MainWidget, Ui_MainWdgt):
 
-    def __init__(self, component_manager):
-        MainWidget.__init__(self, component_manager)
-        QtGui.QMainWindow.__init__(self)
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         # Qt designer does not allow setting multiple shortcuts per action.
         self.actionDeleteCurrentCard.setShortcuts\
@@ -31,7 +32,7 @@ class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
         if event.type() == QtCore.QEvent.LanguageChange:
             self.retranslateUi(self)
         else:
-            QtGui.QMainWindow.changeEvent(self, event)
+            QtWidgets.QMainWindow.changeEvent(self, event)
 
     def createPopupMenu(self):
         # Don't create a silly popup menu saying ('toolBar').
@@ -42,14 +43,33 @@ class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
         self._store_state()
 
     def activate(self):
+        MainWidget.activate(self)
         state = self.config()["main_window_state"]
         if state:
             self.restoreGeometry(state)
+        # Dynamically fill study mode menu.
+        study_modes = [x for x in self.component_manager.all("study_mode")]
+        study_modes.sort(key=lambda x:x.menu_weight)
+        study_mode_group = QtWidgets.QActionGroup(self)
+        self.study_mode_for_action = {}
+        for study_mode in study_modes:
+            action = QtWidgets.QAction(study_mode.name, self)
+            action.setCheckable(True)
+            if self.config()["study_mode"] == study_mode.id:
+                action.setChecked(True)
+            study_mode_group.addAction(action)
+            self.menu_Study.addAction(action)
+            self.study_mode_for_action[action] = study_mode
+        study_mode_group.triggered.connect(self.change_study_mode)
+        self.retranslateUi(self)
         self.timer = QtCore.QTimer()
         self.timer.timeout.connect(self.controller_heartbeat)
         self.timer.start(1000)  # 1 sec.
-        self.start_review()
-        self.retranslateUi(self)
+
+    def change_study_mode(self, action):
+        action.setChecked(True)
+        study_mode = self.study_mode_for_action[action]
+        self.controller().set_study_mode(study_mode)
 
     def controller_heartbeat(self):
         # Need late binding to allow for inheritance.
@@ -60,7 +80,7 @@ class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
         self.setWindowTitle(text)
 
     def top_window(self):
-        for widget in QtGui.QApplication.topLevelWidgets():
+        for widget in QtWidgets.QApplication.topLevelWidgets():
             if not widget.__class__.__name__.startswith("Q") and \
                 widget.__class__.__name__ != "MainWdgt" and \
                 widget.isVisible() == True:
@@ -68,27 +88,42 @@ class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
         return self
 
     def show_information(self, text):
-        QtGui.QMessageBox.information(self.top_window(), _("Mnemosyne"),
-            text, _("&OK"))
+        QtWidgets.QMessageBox.information(self.top_window(), _("Mnemosyne"),
+            text)
 
     def show_question(self, text, option0, option1, option2):
-        return QtGui.QMessageBox.question(self.top_window(),  _("Mnemosyne"),
-            text, option0, option1, option2, 0, -1)
+        dialog = QtWidgets.QMessageBox(self.top_window())
+        dialog.setIcon(QtWidgets.QMessageBox.Question)
+        dialog.setWindowTitle(_("Mnemosyne"))
+        dialog.setText(text)
+        button0 = dialog.addButton(option0, QtWidgets.QMessageBox.ActionRole)
+        button1 = dialog.addButton(option1, QtWidgets.QMessageBox.ActionRole)
+        if option2:
+            button2 = dialog.addButton(option2, QtWidgets.QMessageBox.ActionRole)
+        dialog.exec_()
+        if dialog.clickedButton() == button0:
+            result = 0
+        elif dialog.clickedButton() == button1:
+            result = 1
+        elif dialog.clickedButton() == button2:
+            result = 2
+        return result
 
     def show_error(self, text):
-        QtGui.QMessageBox.critical(self.top_window(), _("Mnemosyne"), text,
-            _("&OK"), "", "", 0, -1)
+        QtWidgets.QMessageBox.critical(self.top_window(), _("Mnemosyne"), text)
 
     def default_font_size(self):
-        return QtGui.qApp.font().pointSize()
+        return QtWidgets.qApp.font().pointSize()
 
     def get_filename_to_open(self, path, filter, caption=""):
-        return unicode(QtGui.QFileDialog.\
-            getOpenFileName(self, caption, path, filter))
+        filename, _ = QtWidgets.QFileDialog.\
+            getOpenFileName(self, caption, path, filter)
+        return filename
 
     def get_filename_to_save(self, path, filter, caption=""):
-        return unicode(QtGui.QFileDialog.\
-            getSaveFileName(self, caption, path, filter))
+        filename, _ = QtWidgets.QFileDialog.\
+            getSaveFileName(self, caption, path, filter)
+        return filename
 
     def set_status_bar_message(self, text):
         self.status_bar.showMessage(text)
@@ -98,7 +133,8 @@ class MainWdgt(QtGui.QMainWindow, Ui_MainWdgt, MainWidget):
             self.progress_bar.close()
             self.progress_bar = None
         if not self.progress_bar:
-            self.progress_bar = QtGui.QProgressDialog(self.top_window())
+            self.progress_bar = QtWidgets.QProgressDialog(self.top_window())
+            self.progress_bar.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
             self.progress_bar.setWindowFlags(QtCore.Qt.Dialog \
                 | QtCore.Qt.CustomizeWindowHint \
                 | QtCore.Qt.WindowTitleHint \
