@@ -4,7 +4,7 @@
 
 import sys
 
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 
 from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.pyqt_ui.ui_sync_dlg import Ui_SyncDlg
@@ -31,26 +31,24 @@ class SyncThread(QtCore.QThread):
 
     """
 
-    information_signal = QtCore.pyqtSignal(QtCore.QString)
-    error_signal = QtCore.pyqtSignal(QtCore.QString)
-    question_signal = QtCore.pyqtSignal(QtCore.QString, QtCore.QString,
-        QtCore.QString, QtCore.QString)
-    set_progress_text_signal = QtCore.pyqtSignal(QtCore.QString)
+    # TODO: migrate to qt_worker_thread mechanism.
+    information_signal = QtCore.pyqtSignal(str)
+    error_signal = QtCore.pyqtSignal(str)
+    question_signal = QtCore.pyqtSignal(str, str, str, str)
+    set_progress_text_signal = QtCore.pyqtSignal(str)
     set_progress_range_signal = QtCore.pyqtSignal(int)
     set_progress_update_interval_signal = QtCore.pyqtSignal(int)
     increase_progress_signal = QtCore.pyqtSignal(int)
     set_progress_value_signal = QtCore.pyqtSignal(int)
     close_progress_signal = QtCore.pyqtSignal()
 
-    def __init__(self, mnemosyne, server, port, username, password):
-        QtCore.QThread.__init__(self)
+    def __init__(self, mnemosyne, server, port, username, password, **kwds):
+        super().__init__(**kwds)
         self.mnemosyne = mnemosyne
         self.server = server
         self.port = port
         self.username = username
         self.password = password
-        # A fast moving progress bar seems to cause crashes on Windows.
-        self.show_numeric_progress_bar = (sys.platform != "win32")
 
     def run(self):
         try:
@@ -98,30 +96,25 @@ class SyncThread(QtCore.QThread):
         self.set_progress_text_signal.emit(text)
 
     def set_progress_range(self, maximum):
-        if self.show_numeric_progress_bar:
-            self.set_progress_range_signal.emit(maximum)
+        self.set_progress_range_signal.emit(maximum)
 
     def set_progress_update_interval(self, value):
-        if self.show_numeric_progress_bar:
-            self.set_progress_update_interval_signal.emit(value)
+        self.set_progress_update_interval_signal.emit(value)
 
     def increase_progress(self, value):
-        if self.show_numeric_progress_bar:
-            self.increase_progress_signal.emit(value)
+        self.increase_progress_signal.emit(value)
 
     def set_progress_value(self, value):
-        if self.show_numeric_progress_bar:
-            self.set_progress_value_signal.emit(value)
+        self.set_progress_value_signal.emit(value)
 
     def close_progress(self):
         self.close_progress_signal.emit()
 
 
-class SyncDlg(QtGui.QDialog, Ui_SyncDlg, SyncDialog):
+class SyncDlg(QtWidgets.QDialog, SyncDialog, Ui_SyncDlg):
 
-    def __init__(self, component_manager):
-        SyncDialog.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+    def __init__(self, **kwds):
+        super().__init__(**kwds)
         self.setupUi(self)
         self.setWindowFlags(self.windowFlags() \
             | QtCore.Qt.WindowMinMaxButtonsHint)
@@ -147,18 +140,26 @@ class SyncDlg(QtGui.QDialog, Ui_SyncDlg, SyncDialog):
     def activate(self):
         self.exec_()
 
+    def _store_state(self):
+        self.config()["sync_dlg_state"] = self.saveGeometry()
+
+    def closeEvent(self, event):
+        # Generated when clicking the window's close button.
+        self._store_state()
+
     def accept(self):
         # Store input for later use.
-        server = unicode(self.server.text())
+        server = self.server.text()
         port = self.port.value()
-        username = unicode(self.username.text())
-        password = unicode(self.password.text())
+        username = self.username.text()
+        password = self.password.text()
         self.config()["server_for_sync_as_client"] = server
         self.config()["port_for_sync_as_client"] = port
         self.config()["username_for_sync_as_client"] = username
         self.config()["password_for_sync_as_client"] = password
         self.config()["check_for_edited_local_media_files"] = \
             self.check_for_edited_local_media_files.isChecked()
+        self._store_state()
         # Prevent user from interrupting a sync.
         self.can_reject = False
         self.ok_button.setEnabled(False)
@@ -189,10 +190,11 @@ class SyncDlg(QtGui.QDialog, Ui_SyncDlg, SyncDialog):
 
     def reject(self):
         if self.can_reject:
-            QtGui.QDialog.reject(self)
+            self._store_state()
+            QtWidgets.QDialog.reject(self)
 
     def finish_sync(self):
-        QtGui.QDialog.accept(self)
+        QtWidgets.QDialog.accept(self)
 
     def threaded_show_information(self, message):
         global answer
